import { RRule, RRuleSet, rrulestr } from "rrule";
import { getHomeUserSchedule1 } from "../../api/request";
import { getDateMap, getExcludeDate, getNextDateList, getWeekNumber, getBlankList } from './schedule.js';
import {
  getFormatDate,
  throttle,
  padZero
} from "../../utils/utils";
const currentDate = new Date();
let minYear = 2019;
let maxYear = currentDate.getFullYear();
const DateMap = getDateMap(minYear, maxYear);
const weekList = ["周日", "周一", "周二", "周三", "周四", "周五", "周六"];


import create from "dd-store";
create.Page({
  data: {
    $data: null,
    scheduleList: [],
    todayStr: "",
    show: false,
    userMsg: {},
    loading: true
  },
  onLoad() {
    this.resetPagination();
    this.getDate();
    this.setData({
      userMsg: {
        headUrl: getApp().globalData.avatar || "",
        userName: getApp().globalData.name
      }
    });
    this.$store.locationSchedule = () => {
      this.locationScheduleList();
    };
  },
  resetPagination() {
    this.pageSize = 20;
    this.totalPages = 1;
    this.currentPage = 1;
    this.firstPage = 1;
    this.scheduleList = [];
  },
  getPagination(scheduleList = []) {
    this.totalPages = Math.ceil(scheduleList.length / this.pageSize);
    for (let i = 0; i < scheduleList.length; i++) {
      if (scheduleList[i].dateStr == currentDate.toLocaleDateString()) {
        this.firstPage = Math.floor(i / this.pageSize);
        this.currentPage = this.firstPage + 1;
        break
      }
    }
  },
  getDate: throttle(function (e) {
    getHomeUserSchedule1({
      startTime: "2020-01-01 00:00:00",
      endTime: "2025-12-30 23:59:59"
    }).then(res => {

      this.scheduleList = this.getScheduleList(res.data.data)
      this.getPagination(this.scheduleList);
      // 第一次加载
      this.setData({
        scheduleList: this.scheduleList.slice(this.firstPage * this.pageSize, this.currentPage * this.pageSize),
        todayStr: currentDate.toLocaleDateString(),
        loading: false
      });
    });
  }, 1000),
  nextDetail: throttle(function (e) {
    let item = e.target.dataset.item;
    dd.navigateTo({
      url: `./../meetingDetail/meetingDetail?scheduleItem=${JSON.stringify(
        item
      )}`
    });
  }, 1000),
  closePop() {
    this.setData({
      show: false
    });
  },
  changesidebar() {
    this.setData({
      show: true
    });
  },
  preventSort() {
    return false;
  },

  //生成DateMap值
  setDateMapValue(startTime, endTime, item) {
    const pushItem = {
      endTime: endTime,
      startTime: startTime,
      planDate: getFormatDate(startTime, "yyyyMMdd", "-"),
      thisDayStartTime: getFormatDate(startTime, 'HH:mm'),
      thisDayEndTime: getFormatDate(endTime, 'HH:mm'),
      scheduleTemplateId: item.id,
      confirmAttendance: item.confirmAttendance,
      title: item.title,
    };
    if (DateMap.has(startTime.toLocaleDateString())) {
      DateMap.get(startTime.toLocaleDateString()).push(pushItem)
    }
    // 如果是跨天会议，需要手动在今天以后的其他天 添加会议日程  
    const nextDateList = getNextDateList(startTime, endTime);
    if (nextDateList.length > 0) {
      for (let i = 0; i < nextDateList.length; i++) {
        if (DateMap.has(nextDateList[i])) {
          DateMap.get(nextDateList[i]).push({
            ...pushItem,
            isDaySpan: true,
            isfirstDayOrEndDay: i === 0 ? "0" : i === nextDateList.length ? "2" : "1", // 0 第一天，1中间的天，2结束的天
            isFewDays: i + 1,
            duration: nextDateList.length + 1,
            isBeOverdue: currentDate.getTime() > endTime.getTime() ? true : false
          });
        }
      }
    }
  },
  // 根据接口返回数据生成填充DateMap
  setDateMapByResponse(response) {
    if (response) {
      const repeatIdList = []; //重复会议的ID
      // 处理重复会议 templateList
      response.templateList.forEach(item => {
        repeatIdList.push(item.id)
        // 生成rrules规则
        const initialTimeList = item.initialTime.split('-');
        const startTimeList = item.startTime.split(":");
        const DTSTART = `${initialTimeList[0]}${initialTimeList[1]}${initialTimeList[2]}T${startTimeList[0]}${startTimeList[1]}00Z`;
        const rule = RRule.fromString(
          `${item.recurrenceModel.recurrenceRule};DTSTART=${DTSTART}`
        );
        // 需要剔除某一天
        if (item.excludePlanDates) {
          const excludeData = getExcludeDate(item.excludePlanDates.split(","));
          rule.all().forEach(startTime => {
            //修正时间（时间有8小时时差）
            startTime.setHours(startTime.getHours() - 8);
            // 生成结束时间
            const endTime = new Date(startTime);
            endTime.setMinutes(endTime.getMinutes() + item.recurrenceModel.duration);
            // 剔除规则
            if (startTime.getTime() < excludeData.minFutureTime &&
              excludeData.excludeDateList.indexOf(getFormatDate(startTime, "yyyyMMdd", "-")) === -1) {
              this.setDateMapValue(startTime, endTime, item);
            }
          });
        }
        // 正常生成数据
        else {
          rule.all().forEach(startTime => {
            //修正时间（时间有8小时时差）
            startTime.setHours(startTime.getHours() - 8);
            // 生成结束时间
            const endTime = new Date(startTime);
            endTime.setMinutes(endTime.getMinutes() + item.recurrenceModel.duration);
            this.setDateMapValue(startTime, endTime, item);
          });
        }
      });

      // 处理单次会议 scheduleList
      response.scheduleList.forEach(item => {

        if (!item.scheduleTemplateId) {
          this.setDateMapValue(new Date(item.startTime.replace(/-/g, '/')), new Date(item.endTime.replace(/-/g, '/')), item);
        }
        // 有scheduleTemplateId则是虚拟会议转成实体会议，需要替换rrule生成的会议记录
        else if (repeatIdList.includes(item.scheduleTemplateId)) {
          const list = DateMap.get(new Date(item.planDate.replace(/-/g, '/')).toLocaleDateString()) || [];
          list.forEach((o, index) => {
            if (o.scheduleTemplateId == item.scheduleTemplateId) {
              list.splice(index, 1)
            }
          });
          this.setDateMapValue(new Date(item.startTime.replace(/-/g, '/')), new Date(item.endTime.replace(/-/g, '/')), item);
        }
      });
    }
  },

  //日程数据
  getScheduleList(response) {
    this.setDateMapByResponse(response);
    // 取出DateMap的value值平铺到DateList
    const DateList = [];
    DateMap.forEach(function (value, key) {
      const keyDate = new Date(key);
      const year = keyDate.getFullYear();
      const month = keyDate.getMonth();
      const date = keyDate.getDate();
      const day = keyDate.getDay();

      // 生成年
      if (month === 0 && date === 1) {
        DateList.push({
          type: "year",
          value: year,
          dateStr: `${year}`
        });
      }
      //生成月
      if (date === 1) {
        DateList.push({
          type: "month",
          value: month + 1,
          dateStr: `${year}/${month + 1}`
        });
      }
      //生成周
      if (day === 0) {
        const rangeDate = new Date(year, month, date + 6);
        const rangeMonth2 = new Date(rangeDate).getMonth();
        const rangeDay2 = new Date(rangeDate).getDate();
        DateList.push({
          type: "week",
          value: `第${getWeekNumber(year, month, date)}周，${month + 1}月${date}日 - ${
            rangeMonth2 == month ? "" : rangeMonth2 + 1 + "月"
            }${rangeDay2}日`,
          dateStr: `${year}/${month + 1}/${date}-week`
        });
      }
      // 有日程长度
      if (value.length !== 0) {
        value.forEach((item, index) => {
          if (index === 0) {
            DateList.push({
              type: key !== currentDate.toLocaleDateString() ? "day" : 'today',
              week: weekList[day],
              hasDateLabel: padZero(date),
              value: item,
              dateStr: `${key}`
            });
          } else {
            DateList.push({
              type: "day",
              value: item,
              dateStr: `${key}/${item.id || item.scheduleTemplateId}`
            });
          }
        });
      } else if (value.length === 0 && key === currentDate.toLocaleDateString()) {
        DateList.push({
          type: "thisDay",
          value: "",
          hasDateLabel: padZero(date),
          week: weekList[day],
          dateStr: key
        });
      }
    });
    return DateList;
  },

  lower() {
    if (this.currentPage < this.totalPages) {
      this.currentPage++;
      this.setData({
        scheduleList: this.scheduleList.slice(this.firstPage * this.pageSize, this.currentPage * this.pageSize)
      });
    } else {
      maxYear++;
      this.data.scheduleList.push(...getBlankList(maxYear));
      this.setData({
        scheduleList: this.data.scheduleList
      })
    }
  },
  onPullDownRefresh() {
    if (this.firstPage > 0) {
      this.firstPage--;
      this.setData({
        todayStr: '',
        scheduleList: this.scheduleList.slice(this.firstPage * this.pageSize, this.currentPage * this.pageSize)
      }, () => {
        this.setData({
          todayStr: this.scheduleList[(this.firstPage + 1) * this.pageSize].dateStr
        });
        dd.stopPullDownRefresh()
      });
    } else {
      minYear--;
      this.data.scheduleList.unshift(...getBlankList(minYear));
      this.setData({
        todayStr: '',
        scheduleList: this.data.scheduleList
      }, () => {
        this.setData({
          // 我已经算不清楚了
          // todayStr: this.data.scheduleList[(2019 - minYear) * 76].dateStr
        });
        dd.stopPullDownRefresh()
      })
    }
  },
  nextPage() {
    dd.navigateTo({ url: "./../createMeeting/createMeeting" });
  },
  locationScheduleList() {
    this.setData(
      {
        todayStr: ""
      },
      () => {
        this.setData({
          todayStr: currentDate.toLocaleDateString()
        });
        this.$store.data.isLocation = false;
        this.update();
      }
    );
  }
});
